##################################################
# Copyright (C) 2018, All rights reserved.
##################################################

from __future__ import print_function
import argparse
import os
import sys

from pyprelude.file_system import make_path
from pysimplevcs.git import Git

from ignoregen import __description__, __project_name__, __version__
from ignoregen.config import Config

def _remove_prefix(prefix, s):
    if s.startswith(prefix):
        return s[len(prefix):]
    else:
        return None

def _get_names_to_ignore(search_dir, truncate=None):
    git = Git(search_dir)
    dir_names = set()
    file_names = set()
    for line in git.status("--ignored").splitlines():
        temp = _remove_prefix("#", line)
        if temp is None:
            temp = _remove_prefix("\t", line)
            if temp is None: continue

        temp = temp.strip()
        if len(temp) == 0: continue

        parts = temp.split("/")[:truncate]
        if len(parts[-1]) == 0: parts.pop()

        name = "/".join(parts)
        path = make_path(search_dir, name)
        if os.path.isfile(path) or os.path.islink(path):
            file_names.add(name)
        if os.path.isdir(path):
            dir_names.add(name)

    return dir_names, file_names

def _do_gen(config, args):
    dir_names, file_names = _get_names_to_ignore(os.path.expanduser("~"), args.truncate)
    print("# Automatically generated by {} version {}".format(__project_name__, __version__))
    if args.truncate is not None:
        print("# --truncate={}".format(args.truncate))
    print()

    print("# Directories")
    for n in sorted(dir_names):
        print("/{}/".format(n))
    print()

    print("# Files and symlinks")
    for n in sorted(file_names):
        print("/{}".format(n))

def _main(argv=None):
    if argv is None:
        argv = sys.argv[1:]

    config_dir = os.path.abspath(os.path.expanduser(os.environ.get("IGNOREGEN_DIR", "~/.ignoregen")))
    config = Config(config_dir)

    parser = argparse.ArgumentParser(prog=__project_name__, description=__description__)
    parser.add_argument("--version", action="version", version="{} version {}".format(__project_name__, __version__))

    subparsers = parser.add_subparsers(help="subcommand help")

    gen_parser = subparsers.add_parser("gen", help="Generate Git ignore rules")
    gen_parser.set_defaults(func=_do_gen)
    gen_parser.add_argument("--truncate", "-t", type=int, default=None, help="Truncate paths to specified level")

    args = parser.parse_args(argv)
    args.func(config, args)

if __name__ == "__main__":
    _main()



##################################################
# Copyright (C) 2017, All rights reserved.
##################################################

import os
import re

from setuptools import find_packages, setup

def _read_properties():
    init_path = os.path.abspath(os.path.join("ignoregen", "__init__.py"))
    regex = re.compile("^\\s*__(?P<key>.*)__\\s*=\\s*\"(?P<value>.*)\"\\s*$")
    with open(init_path, "rt") as f:
        props = {}
        for line in f.readlines():
            m = regex.match(line)
            if m is not None:
                props[m.group("key")] = m.group("value")

    return props

props = _read_properties()
version = props["version"]
description = props["description"]

setup(
    name="ignoregen",
    version=version,
    description=description,
    setup_requires=["setuptools-markdown"],
    long_description_markdown_filename="README.md",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 2.7",
    ],
    url="https://github.com/rcook/ignoregen",
    author="Richard Cook",
    author_email="rcook@rcook.org",
    license="MIT",
    packages=find_packages(),
    install_requires=[
        "pyprelude",
        "pysimplevcs"
    ],
    entry_points={
        "console_scripts": [
            "ignoregen = ignoregen.__main__:_main"
        ]
    },
    include_package_data=True,
    test_suite="ignoregen.tests.suite",
    zip_safe=False)



##################################################
# Copyright (C) 2018, All rights reserved.
##################################################

from __future__ import print_function
import doctest
import importlib
import pkgutil
import unittest

def _get_module_names(package_name):
    return ["{}.{}".format(package_name, name) for _, name, _ in pkgutil.walk_packages([package_name])]

def _get_test_suite(module_name):
    try:
        return doctest.DocTestSuite(importlib.import_module(module_name))
    except ValueError:
        return

def suite():
    suite = unittest.TestSuite()
    for module_name in _get_module_names("ignoregen"):
        tests = _get_test_suite(module_name)
        if tests:
            suite.addTests(tests)

    return suite

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite())
